/* ***********************************************************************
**
**	lcd.c
**
**  Copyright (C) 2005  Christian Kranz
**
**  Siemens S65 Display Control
**
**	Minor changes for a digital picture frame by M.Bode 2007
**
*********************************************************************** */

#include "lcd.h"
#include "main.h"
#define NOP() asm volatile ("nop" ::)


const char init1[] PROGMEM = {
0xFD, 0xFD, 0xFD, 0xFD,
0xEF, 0x00, 0xEE, 0x04, 0x1B, 0x04, 0xFE, 0xFE, 0xFE, 0xFE, 0xEF, 0x90, 0x4A, 0x04,
0x7F, 0x3F, 0xEE, 0x04, 0x43, 0x06 
};

const char init2[] PROGMEM = {
0xEF, 0x90,  0x09, 0x83,  0x08, 0x00,  0x0B, 0xAF,  0x0A, 0x00,  0x05, 0x00,  0x06, 0x00,
0x07, 0x00,  0xEF, 0x00,  0xEE, 0x0C,  0xEF, 0x90,  0x00, 0x80,  0xEF, 0xB0,  0x49, 0x02,
0xEF, 0x00,  0x7F, 0x01,  0xE1, 0x81,  0xE2, 0x02,  0xE2, 0x76,  0xE1, 0x83
};

const char init3[] PROGMEM = {
0x80, 0x01
};

void mswait(uint16_t ms)
{
  uint16_t del;
  while(ms--)
  {
    for(del = 0; del < 800; del++) //1000
      {    NOP();}
  }
}

void lcd_wrdata(char data)
{
	char k;
	LCD_PORT &= ~_BV(LCD_CS);							// select Display
	LCD_PORT &= ~_BV(LCD_RS);							// clear RS line

	for(k=0;k<8;k++)
	{
		if(data & 0x80)		LCD_PORT |= _BV(LCD_DAT);	// Set Bit
		else				LCD_PORT &= ~_BV(LCD_DAT);	// Clear Bit
		LCD_PORT |= _BV(LCD_CLK);						// Set Clock	
		data <<= 1;
		LCD_PORT &= ~_BV(LCD_CLK);						// Clear Clock
	}
	
	LCD_PORT |= _BV(LCD_CS);							// deselect Display
}

void lcd_wrcmd(char data)
{
	char k;
	LCD_PORT &= ~_BV(LCD_CS);							// select Display
	LCD_PORT |= _BV(LCD_RS);							// set RS line

	for(k=0;k<8;k++)
	{
		if(data & 0x80)		LCD_PORT |= _BV(LCD_DAT);	// Set Bit
		else				LCD_PORT &= ~_BV(LCD_DAT);	// Clear Bit
		LCD_PORT |= _BV(LCD_CLK);						// Set Clock	
		data <<= 1;
		LCD_PORT &= ~_BV(LCD_CLK);						// Clear Clock
	}
		
	LCD_PORT |= _BV(LCD_CS);							// deselect Display
}

void lcd_clrscr(void)
{
	uint16_t cnt;
	lcd_wrcmd(0xEF);
	lcd_wrcmd(0x90);
	lcd_wrcmd(0x05);
	lcd_wrcmd(0x00);
	lcd_wrcmd(0x06);
	lcd_wrcmd(0x00);
	lcd_wrcmd(0x07);
	lcd_wrcmd(0x00);
	for(cnt=0;cnt<(132*176);cnt++)
	{
		lcd_wrdata(0xff);
		lcd_wrdata(0xff);
	}
}

void lcd_init(void)
{
	LCD_PORT |= _BV(LCD_CS);	 	
	LCD_DIR |= _BV(LCD_CS);		 	
	port_init_io();
	
	wrtab(init1, 24);
	mswait(7);
	wrtab(init2, 40);
	mswait(50);
	wrtab(init3, 2);
	
	LCD_PORT &= ~_BV(LCD_CS);	// select Display
	
	lcd_wrcmd(0xEF);
	lcd_wrcmd(0x90);
	lcd_wrcmd(0x00);
	lcd_wrcmd(0x00);
}

void wrtab(PGM_P data, char len)
{
	char c;
	while(len--)
	{
		c = pgm_read_byte(data++);
		lcd_wrcmd(c);
	}
}

void port_init_io(void)
{
	LCD_PORT &= ~_BV(LCD_RESET);
	LCD_DIR |= _BV(LCD_RESET);	
	LCD_DIR |= _BV(LCD_DAT);	 	
	LCD_PORT |= _BV(LCD_CS);	 	
	LCD_DIR |= _BV(LCD_CS);		 	

	LCD_PORT &= ~_BV(LCD_CLK);	 
	LCD_DIR |= _BV(LCD_CLK);	 	

	LCD_PORT |= _BV(LCD_RS);	 	
	LCD_DIR |= _BV(LCD_RS);		 	
	mswait(50);
	
	LCD_PORT |= _BV(LCD_RESET);		
	
	mswait(10);
}
